#include "PCSPE.h"
#include "GUI.h"



extern void* hInstance;

inline HINSTANCE GetInstance() { return (HINSTANCE)hInstance; }



GUI::GUI(AudioEffect* effect)
{
	WNDCLASS wc;
	char tempstr[32];

	pcspe=(PCSPE*)effect;

	rect.left  =0;
	rect.top   =0;
	rect.right =GUI_WINDOW_WDT;
	rect.bottom=GUI_WINDOW_HGT;

	sprintf(tempstr,"PCSPE%08x",(long)this);

	wc.style=CS_GLOBALCLASS;
	wc.lpfnWndProc=WndProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=0;
	wc.hInstance=GetInstance();
	wc.hIcon=0;
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground=GetSysColorBrush(COLOR_BTNFACE);
	wc.lpszMenuName=0;
	wc.lpszClassName=tempstr; 

	RegisterClass(&wc);
}



GUI::~GUI()
{
	char tempstr[32];

	sprintf(tempstr,"PCSPE%08x",(long)this);

	UnregisterClass(tempstr,GetInstance());
}
	
	
	
bool GUI::open(void* ptr)
{
	const char* const int_freq_name[]={"18Hz","50Hz","60Hz","70Hz","120Hz","240Hz","480Hz"};
	HWND parent;
	HINSTANCE hinst;
	char tempstr[32];
	int i,x,y,w,h,style;

	AEffEditor::open(ptr);

	parent=HWND(ptr);
	hinst=GetInstance();

	sprintf(tempstr,"PCSPE%08x",(long)this);

	hWndForm=CreateWindow(tempstr,"PCSPE",WS_CHILD|WS_VISIBLE,0,0,(rect.right-rect.left),(rect.bottom-rect.top),parent,0,hinst,this);

	SetWindowLong(hWndForm,GWLP_USERDATA,(long)this);

	style=WS_CHILD|WS_VISIBLE;

	x=10;
	y=14;
	w=60;
	h=25;

	hWndLabelC=CreateWindow("static","ST_U",style,x,y,100,h,hWndForm,NULL,hinst,NULL); y+=40;
	hWndLabelV=CreateWindow("static","ST_U",style,x,y,w,h,hWndForm,NULL,hinst,NULL); y+=30;
	hWndLabelA=CreateWindow("static","ST_U",style,x,y,w,h,hWndForm,NULL,hinst,NULL); y+=30;
	hWndLabelP=CreateWindow("static","ST_U",style,x,y,w,h,hWndForm,NULL,hinst,NULL); y+=40;
	hWndLabelIF=CreateWindow("static","ST_U",style,x,y,w,h,hWndForm,NULL,hinst,NULL); y+=30;
	hWndLabelAS=CreateWindow("static","ST_U",style,x,y,w,h,hWndForm,NULL,hinst,NULL);

	SetWindowText(hWndLabelC,"MIDI channel:");
	SetWindowText(hWndLabelV,"Volume:");
	SetWindowText(hWndLabelA,"Arp:");
	SetWindowText(hWndLabelP,"Pitch:");
	SetWindowText(hWndLabelIF,"Int. freq:");
	SetWindowText(hWndLabelAS,"Auto arp:");

	x=110;
	y=10;
	w=24;
	h=24;

	style=WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON|BS_OWNERDRAW|BS_NOTIFY;

	for(i=0;i<16;++i)
	{
		sprintf(tempstr,"%i",i+1);

		hWndChannel[i]=CreateWindow("button",tempstr,style,x,y,w,h,hWndForm,NULL,hinst,NULL);

		x+=w+6;
	}

	style=WS_CHILD|WS_VISIBLE|ES_LEFT|WS_BORDER|ES_AUTOHSCROLL;

	x=80;
	y=50;
	w=GUI_WINDOW_WDT-10-x;
	h=25;

	hWndEditV=CreateWindow("edit","0",style,x,y,w,h,hWndForm,NULL,hinst,NULL); y+=30;
	hWndEditA=CreateWindow("edit","0",style,x,y,w,h,hWndForm,NULL,hinst,NULL); y+=30;
	hWndEditP=CreateWindow("edit","0",style,x,y,w,h,hWndForm,NULL,hinst,NULL);

	SendMessage(hWndEditV,EM_SETLIMITTEXT,ENVELOPE_TEXT_LEN,0);
	SendMessage(hWndEditA,EM_SETLIMITTEXT,ENVELOPE_TEXT_LEN,0);
	SendMessage(hWndEditP,EM_SETLIMITTEXT,ENVELOPE_TEXT_LEN,0);

	SetWindowLongPtr(hWndEditV,GWLP_USERDATA,0xdeadf00b);	//Reaper hack to prevent hijacking Space button
	SetWindowLongPtr(hWndEditA,GWLP_USERDATA,0xdeadf00b);
	SetWindowLongPtr(hWndEditP,GWLP_USERDATA,0xdeadf00b);

	x=80;
	y=150;
	w=45;
	h=24;

	style=WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON|BS_OWNERDRAW|BS_NOTIFY;

	for(i=0;i<7;++i)
	{
		hWndIntFreq[i]=CreateWindow("button",int_freq_name[i],style,x,y,w,h,hWndForm,NULL,hinst,NULL);

		x+=w+3;
	}

	x+=10;
	w=50;

	hWndMultiMode=CreateWindow("button","Multi",style,x,y,w,h,hWndForm,NULL,hinst,NULL);

	w=70;
	x=GUI_WINDOW_WDT-w-10-4-h;
	
	hWndLogFile=CreateWindow("button","Log start",style,x,y,w,h,hWndForm,NULL,hinst,NULL);

	x+=w+4;
	w=h;

	hWndLogFileAuto=CreateWindow("button","AC",style,x,y,w,h,hWndForm,NULL,hinst,NULL);

	pcspe->SetUpdateGUI();

	return true;
}



void GUI::close()
{
	int i;

	DestroyWindow(hWndLabelC);
	DestroyWindow(hWndLabelV);
	DestroyWindow(hWndLabelA);
	DestroyWindow(hWndLabelP);
	DestroyWindow(hWndLabelIF);
	DestroyWindow(hWndLabelAS);
	DestroyWindow(hWndEditV);
	DestroyWindow(hWndEditA);
	DestroyWindow(hWndEditP);

	for(i=0;i<16;++i) DestroyWindow(hWndChannel[i]);
	for(i=0;i<7;++i) DestroyWindow(hWndIntFreq[i]);

	DestroyWindow(hWndLogFile);
	DestroyWindow(hWndLogFileAuto);
	DestroyWindow(hWndMultiMode);

	DestroyWindow(hWndForm);

	AEffEditor::close();
}



bool GUI::getRect(ERect **ppErect)
{
   *ppErect=&rect;

   return true;
}



void GUI::Update(void)
{
	int pgm;
	char label[1024],newlabel[1024];

	pgm=pcspe->Program[pcspe->MidiChannel];

	SetWindowText(hWndEditV,pcspe->EnvelopeV[pgm]);
	SetWindowText(hWndEditA,pcspe->EnvelopeA[pgm]);
	SetWindowText(hWndEditP,pcspe->EnvelopeP[pgm]);

	GetWindowText(hWndLogFile,label,GetWindowTextLength(hWndLogFile)+1);

	strcpy(newlabel,pcspe->LogActive?"Log stop":"Log start");

	if(strcmp(label,newlabel))
	{
		SetWindowText(hWndLogFile,newlabel);

		InvalidateRect(hWndLogFile,NULL,true);
	}

	InvalidateRect(hWndMultiMode,NULL,true);
}



LONG WINAPI GUI::WndProc(HWND hWnd,UINT message,WPARAM wParam,LPARAM lParam)
{
	LPDRAWITEMSTRUCT lpDIS;
	OPENFILENAME ofn;
	HWND phWnd;
	HDC phDC;
	GUI *gui;
	char tempstr[16],ext[1024];
	int i,n,len,color;
	bool compile,highlight,noext;

	gui=(GUI*)GetWindowLong(hWnd,GWLP_USERDATA);

	phWnd=(HWND)lParam;
	phDC=(HDC)wParam;

	switch(message)
	{
	case WM_COMMAND:
		{
			if(HIWORD(wParam)==EN_CHANGE)	//any edit has been changed
			{
				compile=false;

				if(phWnd==gui->hWndEditV) { GetWindowText(phWnd,gui->pcspe->EnvelopeV[gui->pcspe->Program[gui->pcspe->MidiChannel]],ENVELOPE_TEXT_LEN); compile=true; }
				if(phWnd==gui->hWndEditA) { GetWindowText(phWnd,gui->pcspe->EnvelopeA[gui->pcspe->Program[gui->pcspe->MidiChannel]],ENVELOPE_TEXT_LEN); compile=true; }
				if(phWnd==gui->hWndEditP) { GetWindowText(phWnd,gui->pcspe->EnvelopeP[gui->pcspe->Program[gui->pcspe->MidiChannel]],ENVELOPE_TEXT_LEN); compile=true; }

				if(compile)
				{
					gui->pcspe->CompileInstruments();

					InvalidateRect(gui->hWndLabelV,NULL,true);
					InvalidateRect(gui->hWndLabelA,NULL,true);
					InvalidateRect(gui->hWndLabelP,NULL,true);
				}

				return 0;
			}

			for(i=0;i<16;++i)
			{
				if(phWnd==gui->hWndChannel[i])
				{
					if(gui->pcspe->MidiChannel!=i)
					{
						gui->pcspe->MidiChannel=i;

						for(i=0;i<16;++i) InvalidateRect(gui->hWndChannel[i],NULL,true);

						gui->pcspe->SetUpdateGUI();
					}

					return 0;
				}
			}

			if(phWnd==gui->hWndLogFileAuto)
			{
				if(HIWORD(wParam)==BN_CLICKED)
				{
					gui->pcspe->LogAuto^=true;

					InvalidateRect(gui->hWndLogFileAuto,NULL,true);

					gui->pcspe->SetUpdateGUI();
				}

				return 0;
			}

			if(phWnd==gui->hWndMultiMode)
			{
				if(HIWORD(wParam)==BN_CLICKED)
				{
					gui->pcspe->MultiMode[gui->pcspe->MidiChannel]^=true;

					InvalidateRect(gui->hWndMultiMode,NULL,true);

					gui->pcspe->SetUpdateGUI();
				}

				return 0;
			}

			for(i=0;i<7;++i)
			{
				if(phWnd==gui->hWndIntFreq[i])
				{
					switch(i)
					{
					case 0: gui->pcspe->FrameRate=18.2f; break;
					case 1: gui->pcspe->FrameRate=50.0f; break;
					case 2: gui->pcspe->FrameRate=60.0f; break;
					case 3: gui->pcspe->FrameRate=70.0f; break;
					case 4: gui->pcspe->FrameRate=120.0f; break;
					case 5: gui->pcspe->FrameRate=240.0f; break;
					case 6: gui->pcspe->FrameRate=480.0f; break;
					}

					for(i=0;i<7;++i) InvalidateRect(gui->hWndIntFreq[i],NULL,true);

					gui->pcspe->SetUpdateGUI();

					return 0;
				}
			}


			if(phWnd==gui->hWndLogFile)
			{
				if(HIWORD(wParam)==BN_CLICKED)
				{
					if(gui->pcspe->LogActive)
					{
						gui->pcspe->LogStop();
					}
					else
					{
						ZeroMemory(&ofn,sizeof(ofn));
						ofn.lStructSize=sizeof(ofn);

						ofn.hwndOwner=hWnd;
						ofn.nFilterIndex=1;
						ofn.nMaxFileTitle=0;
						ofn.Flags=OFN_ENABLESIZING|OFN_OVERWRITEPROMPT;

						ofn.lpstrFileTitle=NULL;
						ofn.lpstrFilter="CSV file\0*.csv\0QBASIC data\0*.bas\0C header\0*.h\0All\0*.*\0";
						ofn.lpstrInitialDir=NULL;
						ofn.lpstrFile=gui->pcspe->LogFileName;
						ofn.nMaxFile=MAX_PATH;
						ofn.lpstrTitle="Save timer log file";

						if(GetOpenFileName(&ofn))
						{
							strcpy(gui->pcspe->LogFileName,ofn.lpstrFile);

							strcpy(ext,ofn.lpstrFile);

							noext=true;

							for(i=strlen(ext)-1;i>=0;--i)
							{
								if(ext[i]=='.')
								{
									strcpy(ext,&ext[i+1]);
									noext=false;
									break;
								}
							}

							if(noext)
							{
								switch(ofn.nFilterIndex)
								{
								case 1: strcat(gui->pcspe->LogFileName,".csv"); strcpy(ext,"csv"); break;
								case 2: strcat(gui->pcspe->LogFileName,".bas"); strcpy(ext,"bas"); break;
								case 3: strcat(gui->pcspe->LogFileName,".h"  ); strcpy(ext,"h"  ); break;
								}
							}

							gui->pcspe->LogFormat=LOG_FORMAT_CSV;

							if(!strcmp(ext,"bas")||!strcmp(ext,"BAS")) gui->pcspe->LogFormat=LOG_FORMAT_QB;
							if(!strcmp(ext,"h")||!strcmp(ext,"H")) gui->pcspe->LogFormat=LOG_FORMAT_H;

							gui->pcspe->LogStart();
						}
					}

					return 0;
				}
			}
		}
		break;

	case WM_KEYDOWN:
		{
		}
		return 0;

	case WM_DRAWITEM:
		{
			lpDIS=(LPDRAWITEMSTRUCT)lParam;
		
			highlight=false;

			if(lpDIS->hwndItem==gui->hWndChannel[gui->pcspe->MidiChannel]) highlight=true;

			if(lpDIS->hwndItem==gui->hWndLogFileAuto&&gui->pcspe->LogAuto) highlight=true;

			if(lpDIS->hwndItem==gui->hWndMultiMode&&gui->pcspe->MultiMode[gui->pcspe->MidiChannel]) highlight=true;

			n=-1;

			if(gui->pcspe->FrameRate==18.2f) n=0;
			if(gui->pcspe->FrameRate==50.0f) n=1;
			if(gui->pcspe->FrameRate==60.0f) n=2;
			if(gui->pcspe->FrameRate==70.0f) n=3;
			if(gui->pcspe->FrameRate==120.0f) n=4;
			if(gui->pcspe->FrameRate==240.0f) n=5;
			if(gui->pcspe->FrameRate==480.0f) n=6;

			if(n>=0&&lpDIS->hwndItem==gui->hWndIntFreq[n]) highlight=true;

			color=highlight?GetSysColor(COLOR_BTNSHADOW):color=GetSysColor(COLOR_BTNFACE);

			SetDCBrushColor(lpDIS->hDC,color);
			SetBkColor(lpDIS->hDC,color);

			SelectObject(lpDIS->hDC,GetStockObject(DC_BRUSH));

			RoundRect(lpDIS->hDC,lpDIS->rcItem.left,lpDIS->rcItem.top,lpDIS->rcItem.right,lpDIS->rcItem.bottom,5,5);

			len=GetWindowTextLength(lpDIS->hwndItem);

			GetWindowTextA(lpDIS->hwndItem,tempstr,len+1);

			lpDIS->rcItem.top+=4;

			DrawTextA(lpDIS->hDC,tempstr,len,&lpDIS->rcItem,DT_CENTER);

			return TRUE;
		}
        break;

	case WM_CTLCOLORSTATIC:
		{
			if(phWnd==gui->hWndLabelV)
			{
				if(gui->pcspe->Instrument[gui->pcspe->Program[gui->pcspe->MidiChannel]].EnvVol.error)
				{
					SetBkColor(phDC,GUI_COLOR_ERROR);

					return (LRESULT)GetSysColorBrush(COLOR_BTNFACE);
				}
			}

			if(phWnd==gui->hWndLabelA)
			{
				if(gui->pcspe->Instrument[gui->pcspe->Program[gui->pcspe->MidiChannel]].EnvArp.error)
				{
					SetBkColor(phDC,GUI_COLOR_ERROR);

					return (LRESULT)GetSysColorBrush(COLOR_BTNFACE);
				}
			}

			if(phWnd==gui->hWndLabelP)
			{
				if(gui->pcspe->Instrument[gui->pcspe->Program[gui->pcspe->MidiChannel]].EnvPitch.error)
				{
					SetBkColor(phDC,GUI_COLOR_ERROR);

					return (LRESULT)GetSysColorBrush(COLOR_BTNFACE);
				}
			}

			SetBkColor(phDC,GetSysColor(COLOR_BTNFACE));

			return (LRESULT)GetSysColorBrush(COLOR_BTNFACE);
		}
	}

	return DefWindowProc(hWnd,message,wParam,lParam);
}